@@ -8,8 +8,8 @@
 ;; Keywords: calendar
 ;; URL: https://github.com/TomoeMami/org-repeat-by-cron.el
 
-;; Package-Version: 1.0.5
-;; Package-Revision: 0f29e2fe3898
+;; Package-Version: 1.1.3
+;; Package-Revision: 6586282bd13d
 ;; Package-Requires: ((emacs "24.4"))
 
 ;; This file is not part of GNU Emacs.
@@ -29,38 +29,45 @@
 
 ;;; Commentary:
 ;; An Org mode task repeater based on Cron expressions
+;; 
 ;; Modified from https://github.com/Raemi/org-reschedule-by-rule.
+;; 
 ;; Key Differences:
 ;; - Uses a cron parser implemented in pure Elisp, with
 ;;   no dependency on the Python croniter package.
 ;; - Replaces the INTERVAL property with a DAY_AND property.
 ;; - Supports toggling between SCHEDULED and DEADLINE timestamps.
+;; - Fully compatible with org-habit; it takes over rescheduling
+;;   while preserving the habit's consistency graph.
 ;; 
 ;; org-repeat-by-cron.el is a lightweight extension for Emacs Org
-;; mode that allows you to repeat tasks based on powerful Cron expressions.
-;; Standard Org mode repeaters (like +1d , ++1w ) are based on
+;; mode that allows you to repeat tasks using the power of Cron expressions.
+;; Standard Org mode repeaters (like +1d , ++1w ) are relative to
 ;; the current SCHEDULED or DEADLINE timestamp. In contrast, this
-;; tool provides a repetition method based on absolute time rules.
-;; You can easily set a task to repeat \"on the last Friday of
-;; every month\" or \"on the first Monday of each quarter\"
-;; without manual date calculations.
-;; A core advantage of this tool is its pure Elisp implementation,
-;; which does not rely on any external programs (like Python's
-;; croniter library), ensuring it works out-of-the-box in Emacs environment.
+;; tool provides repetition based on absolute time rules. You can
+;; easily set a task to repeat "on the last Friday of every month"
+;; or "on the first Monday of each quarter" without manual date
+;; calculations.
+;; 
+;; A core advantage is its pure Elisp implementation, ensuring it
+;; works out-of-the-box in any Emacs environment without external
+;; dependencies.
 ;; 
 ;; Installation
 ;; Install via Melpa :
 ;; #+begin_src elisp
 ;; (use-package org-repeat-by-cron
-;;   :ensure t  ; If the file is already in your load-path
+;;   :ensure t
 ;;   :config
 ;;   (global-org-repeat-by-cron-mode))
 ;; #+end_src
 ;; 
 ;; Usage
 ;; 
-;; To make an Org task repeat according to a Cron rule, 
+;; To make an Org task repeat according to a Cron rule,
 ;; simply add the =REPEAT_CRON= property to its =PROPERTIES= drawer.
+;;
+;; Tip: You do not need to wrap the =REPEAT_CRON= value in quotes.
 ;; 
 ;; Example
 ;; 
@@ -69,112 +76,132 @@
 ;; #+begin_src org
 ;; ,* TODO Weekend Course
 ;; :PROPERTIES:
-;; :REPEAT_CRON: "* * SAT,SUN"
+;; :REPEAT_CRON: * * SAT,SUN
 ;; :END:
 ;; #+end_src
 ;; 
-;; When it is marked as done, it will automatically be scheduled to
-;; a date that meets the conditions. Taking today (December 9, 2025)
-;; as an example, it will be scheduled for this Saturday 
-;; (December 13, 2025):
+;; When marked as =DONE=, it will automatically be rescheduled
+;; to the next matching date. For example, if today is Dec 9, 2025,
+;; it will be scheduled for the coming Saturday (Dec 13, 2025):
 ;; 
 ;; #+begin_src org
 ;; ,* TODO Weekend Course
 ;; SCHEDULED: <2025-12-13 Sat>
 ;; :PROPERTIES:
-;; :REPEAT_CRON: "* * SAT,SUN"
+;; :REPEAT_CRON: * * SAT,SUN
 ;; :REPEAT_ANCHOR: 2025-12-13 Sat
 ;; :END:
 ;; #+end_src
 ;; 
-;; Then, if it is marked as done again, it will calculate the next qualifying
-;; time point based on the =REPEAT_ANCHOR= and the current time, and schedule
-;; it accordingly:
+;; Marking it =DONE= again will calculate the next point based
+;; on the =REPEAT_ANCHOR= and the current time:
 ;; 
 ;; #+begin_src org
 ;; ,* TODO Weekend Course
 ;; SCHEDULED: <2025-12-14 Sun>
 ;; :PROPERTIES:
-;; :REPEAT_CRON: "* * SAT,SUN"
+;; :REPEAT_CRON: * * SAT,SUN
 ;; :REPEAT_ANCHOR: 2025-12-14 Sun
 ;; :END:
 ;; #+end_src
 ;; 
-;; Tip1: If you do not want to specify the hour and minute for the repeat time, 
-;; use the abbreviated 3-segment format.
-;; Tip2: You should not use org-repeat-by-cron and the built-in
-;; Org repeater cookie (e.g., +1w) on the same task.
-
+;; Tip1: If you do not want to specify a specific hour/minute,
+;; use the *3-field shorthand* (discussed below).
+;; Tip2: If you use org-repeat-by-cron with the built-in Org repeater
+;; cookie (e.g., +1w) on the same task, the build-in cookie will
+;; be taken over and preserved, so you can use this package with
+;; org-habit.
+;;
+;; Check README for more usage: https://github.com/TomoeMami/org-repeat-by-cron.el
+;; 
 ;;; Code:
 
 (require 'cl-lib)
 (require 'org)
 
 (defgroup org-repeat-by-cron nil
-  "Customizations for org-repeat-by-cron-mode."
+  "Cron-style task repetition for Org mode.
+
+This group contains options for a system that allows Org tasks to repeat
+according to complex cron-style schedules instead of simple intervals."
   :group 'org)
 
 (defconst org-repeat-by-cron--month-aliases
   '(("jan" . "1") ("feb" . "2") ("mar" . "3") ("apr" . "4") ("may" . "5")
     ("jun" . "6") ("jul" . "7") ("aug" . "8") ("sep" . "9") ("oct" . "10")
     ("nov" . "11") ("dec" . "12"))
-  "Mapping of month name abbreviations to numbers.")
+  "Alist mapping month name abbreviations to their numeric strings.")
 
 (defconst org-repeat-by-cron--dow-aliases
   '(("sun" . "0") ("mon" . "1") ("tue" . "2") ("wed" . "3") ("thu" . "4")
     ("fri" . "5") ("sat" . "6"))
-  "Mapping of day-of-week name abbreviations to numbers (0=Sunday).")
+  "Alist mapping day-of-week abbreviations to their numeric strings.")
 
 (defcustom org-repeat-by-cron-anchor-prop "REPEAT_ANCHOR"
-  "Name of the Org property for the cron repetition anchor timestamp.
+  "Org property name used to store the last calculated repeat timestamp.
 
-This variable holds the string used as the property key to store
-the base timestamp for cron-based repetitions.  When a repeating
-entry is marked as done, the function `org-repeat-by-cron-on-done'
-looks for this property to determine the starting point for
-calculating the next occurrence.  The property's value is then
-updated to the new scheduled time."
+This property acts as the reference point for calculating the next
+repetition occurrence to ensure consistency when a task is completed."
   :group 'org-repeat-by-cron
   :type 'string)
 
 (defcustom org-repeat-by-cron-day-and-prop "REPEAT_DAY_AND"
-  "Name of the Org property to control the day-of-month/day-of-week logic.
+  "Org property name that determines the logic for day matching.
 
-This variable holds the string used as the property key to specify
-how the day-of-month and day-of-week fields of a cron rule are
-combined.  The function `org-repeat-by-cron-on-done' checks for
-this property in the Org entry.
-
-If this property's value is set to the string \"t\", a date must
-satisfy both the day-of-month and day-of-week rules (logical AND).
-Otherwise, a date matches if it satisfies either rule (logical OR)."
+If this property is set to \"t\" in an entry, the next occurrence
+must satisfy both the day-of-month and the day-of-week rules.
+Otherwise, it satisfies either rule (standard cron behavior)."
   :group 'org-repeat-by-cron
   :type 'string)
 
 (defcustom  org-repeat-by-cron-cron-prop "REPEAT_CRON"
-  "Name of the Org property for the cron repetition rule.
+  "Org property name used to store the cron expression for a task.
 
-This variable holds the string used as the property key to store
-the cron repetition rule itself.  The function
-`org-repeat-by-cron-on-done' looks for an Org entry property
-with this key.  The value associated with this property must be a
-valid 3- or 5-field cron string."
+The value should be a standard 5-field cron string or a 3-field
+string (Day Month Day-of-Week) for daily-level precision."
   :group 'org-repeat-by-cron
   :type 'string)
 
 (defcustom org-repeat-by-cron-deadline-prop "REPEAT_DEADLINE"
-  "Name of the Org property to trigger deadline-based repetition.
+  "Org property name defining how deadlines are rescheduled via cron.
 
-This variable holds the string used as a property key.  When the
-function `org-repeat-by-cron-on-done' processes a completed entry,
-it checks for a property with this key.  If the property exists
-and its value is the string \"t\", the function will update the
-entry's `DEADLINE` timestamp to the next calculated time.  In
-this case, any existing `SCHEDULED` timestamp on the entry will be
-removed."
+In an Org entry, the property named by this variable determines
+the logic used to move the deadline forward when a task is
+marked DONE.
+
+If the property named by this variable is set to \"t\",
+the next deadline is calculated using the primary cron expression
+from `org-repeat-by-cron-cron-prop'. This calculation uses the
+entry's current deadline as the anchor time instead of its
+scheduled time. Additionally, the existing `SCHEDULED' timestamp is
+cleared when the task is marked DONE.
+
+Setting the property value to a specific cron expression (e.g.,
+\"0 9 1 * *\") decouples the deadline from the schedule. The
+deadline will be recalculated according to its own rule,
+regardless of when the task is scheduled to start."
   :group 'org-repeat-by-cron
   :type 'string)
 
+(defcustom org-repeat-by-cron-deadline-anchor-prop "REPEAT_DEADLINE_ANCHOR"
+  "Org property name for storing the last calculated deadline time.
+
+This property is automatically created and used only when the property
+specified by `org-repeat-by-cron-deadline-prop' contains a specific
+cron expression. If that property is set to the string \"t\", the
+deadline is recalculated using the primary cron rule instead, and
+this anchor property will not be created or updated."
+  :group 'org-repeat-by-cron
+  :type 'string)
+
+(defcustom org-repeat-by-cron-max-search-year 50
+  "Maximum number of years to look ahead for a matching date.
+
+This prevents infinite loops if a cron expression describes a date
+that is impossible or extremely far in the future."
+  :group 'org-repeat-by-cron
+  :type 'integer)
+
 ;; --- [Date Calculation Helper Functions] ---
 
 (defun org-repeat-by-cron--substitute-aliases (field-str alias-map)
@@ -198,11 +225,14 @@ the transformed string."
                                                number-str result))))))
 
 (defun org-repeat-by-cron--get-dow (day month year)
-  "Return the day of the week for the date.
+  "Return the day of the week for DAY, MONTH, and YEAR.
 
-Specified by DAY, MONTH, and YEAR.  The returned value is an
-integer from 0,7 (Sunday) to 6 (Saturday)."
-  (nth 6 (decode-time (encode-time 0 0 0 day month year))))
+The result is an integer between 0 and 6, where 0 represents
+Sunday, 1 represents Monday, and so on.
+
+The arguments DAY, MONTH, and YEAR must be integers. MONTH
+should be in the range 1--12."
+  (calendar-day-of-week (list month day year)))
 
 (defun org-repeat-by-cron--find-nth-dow-of-month (n target-dow month year)
   "Return the day of the month for the Nth TARGET-DOW in MONTH of YEAR.
@@ -310,25 +340,31 @@ function will parse and include the numeric value (15) in the
 list, but it does not implement the logic for finding the
 nearest weekday.  That responsibility is left to the caller."
   ;; * Means all range.
-  (if (string= field-str "*") (number-sequence min max)
+  (if (string= field-str "*")
+      (number-sequence min max)
     ;; Else, prepare numbers under rules.
-    (let ((values '()))
+    (let (values)
       ;; Split by ','
       (dolist (part (split-string field-str ","))
         (cond
          ;; Step
-         ((string-match "/\\([0-9]+\\)" part)
-          (let* ((step (string-to-number (match-string 1 part)))
-                 (base-str (substring part 0 (match-beginning 0)))
-                 (base-range (if (string= base-str "*") (number-sequence min max)
-                               (let ((r (mapcar #'string-to-number (split-string base-str "-")))) (number-sequence (car r) (cadr r)))))
-                 (result '()))
-            (dolist (i base-range (setq result (nreverse result))) (when (zerop (% (- i min) step)) (push i result)))
-            (setq values (append result values))))
+         ((string-match "\\(.*\\)/\\([0-9]+\\)" part)
+          (let* ((range-str (match-string 1 part))
+                 (step (string-to-number (match-string 2 part)))
+                 (range (if (string= range-str "*")
+                            (cons min max)
+                          (let ((r (split-string range-str "-")))
+                            (cons (string-to-number (car r)) (string-to-number (cadr r)))))))
+            (cl-loop for i from (car range) to (cdr range) by step
+                     do (push i values))))
          ;; Range
-         ((string-match "-" part) (setq values (append (let ((r (mapcar #'string-to-number (split-string part "-")))) (number-sequence (car r) (cadr r))) values)))
+         ((string-match "\\([0-9]+\\)-\\([0-9]+\\)" part)
+          (cl-loop for i from (string-to-number (match-string 1 part))
+                   to (string-to-number (match-string 2 part))
+                   do (push i values)))
          ;; Ignore [0-9]+W , deal with it later
-         ((string-match "\\([0-9]+\\)W" part) (push (string-to-number (match-string 1 part)) values))
+         ((string-match "\\([0-9]+\\)W" part)
+          (push (string-to-number (match-string 1 part)) values))
          ;; Default numbers, save it directly
          (t (push (string-to-number part) values))))
       (sort (cl-delete-duplicates values) '<))))
@@ -353,7 +389,7 @@ The DOM-RULE supports several special formats:
 
 The DOW-RULE also supports special formats, which can be part of
 a comma-separated list:
-- \"L5\": The last Friday of the month.  Both 0 and 7 represent Sunday.
+- \"5L\": The last Friday of the month.  Both 0 and 7 represent Sunday.
 - \"5#3\": The third Friday of the month."
   (let ((dow (org-repeat-by-cron--get-dow day month year)) dom-match-p dow-match-p)
     ;; 1. Evaluate Day-of-Month (DOM) rule
@@ -411,7 +447,7 @@ a comma-separated list:
 The function parses the five-field CRON-STRING to find the next valid
 occurrence, starting the search from the minute immediately after
 START-TIME.  It returns the result as an Emacs time value, or nil
-if no match is found before the year 2099.
+if no match is found within `org-repeat-by-cron-max-search-year'.
 
 CRON-STRING must contain five space-separated fields:
 1. Minute (0-59)
@@ -426,13 +462,14 @@ with comma \",\"). If DAY-AND is nil (by default), a date need only
 satisfy one of the conditions (OR). If it is t, a date must
 satisfy both conditions (AND)."
   (let* ((cron-parts (split-string cron-string "[ \t]+" t))
-         (minute-rule (nth 0 cron-parts))
-         (hour-rule (nth 1 cron-parts))
+         (minute-list   (org-repeat-by-cron--expand-field (nth 0 cron-parts) 0 59))
+         (hour-list  (org-repeat-by-cron--expand-field (nth 1 cron-parts) 0 23))
          (dom-rule (nth 2 cron-parts))
          ;; Convert abbreviations to numbers
          (month-rule (org-repeat-by-cron--substitute-aliases (nth 3 cron-parts) org-repeat-by-cron--month-aliases))
+         (month-list (org-repeat-by-cron--expand-field month-rule 1 12))
          (dow-rule (org-repeat-by-cron--substitute-aliases (nth 4 cron-parts) org-repeat-by-cron--dow-aliases))
-         (end-time-year 2099)
+         (end-time-year (+ (nth 5 (decode-time)) org-repeat-by-cron-max-search-year))
          ;; Start searching from the next minute
          (time (time-add start-time 60))
          (decoded-time (decode-time time)))
@@ -447,7 +484,7 @@ satisfy both conditions (AND)."
         ;; Iterate by year
         (cl-loop for year from start-year to end-time-year do
                  ;; Iterate through each month that matches the condition
-                 (dolist (month (org-repeat-by-cron--expand-field month-rule 1 12))
+                 (dolist (month month-list)
                    ;; If this month is greater than or equal to the starting month
                    (when (>= month (if (= year start-year) start-mon 1))
                      ;; Iterate through days of the month, starting from start day if it's the starting month
@@ -457,12 +494,12 @@ satisfy both conditions (AND)."
                               (when (org-repeat-by-cron--day-fields-match-p dom-rule dow-rule day month year day-and)
                                 (let ((current-hour-start (if (and (= year start-year) (= month start-mon) (= day start-day))
                                                               start-hour 0)))
-                                  (dolist (hour (org-repeat-by-cron--expand-field hour-rule 0 23))
+                                  (dolist (hour hour-list)
                                     ;; When hour satisfies the rule, check minutes
                                     (when (>= hour current-hour-start)
                                       (let ((current-min-start (if (and (= year start-year) (= month start-mon) (= day start-day) (= hour start-hour))
                                                                    start-min 0)))
-                                        (dolist (minute (org-repeat-by-cron--expand-field minute-rule 0 59))
+                                        (dolist (minute minute-list)
                                           (when (>= minute current-min-start)
                                             (cl-return-from finder
                                               (encode-time 0 minute hour day month year)))))))))))))
@@ -474,11 +511,12 @@ satisfy both conditions (AND)."
 The function returns 3 if RULE contains three space-separated fields,
 and 5 if it contains five.  In all other cases, it returns nil.
 Whitespace at the beginning or end of RULE is ignored."
-  (let* ((parts (split-string (string-trim rule) "[ \t]+" t))
-         (n (length parts)))
-    (cond ((= n 3) 3)
-          ((= n 5) 5)
-          (t nil))))
+  (if rule
+      (let* ((parts (split-string (string-trim rule) "[ \t]+" t))
+             (n (length parts)))
+        (cond ((= n 3) 3)
+              ((= n 5) 5)
+              (t nil)))))
 
 (defun org-repeat-by-cron--normalize-cron-rule (rule)
   "Normalize the cron RULE string to a 5-field format.
@@ -495,7 +533,7 @@ error message and returns nil."
       (3 (concat "0 0 " (string-trim rule)))
       (5 (string-trim rule))
       (_ (progn
-           (message "org-repeat-by-cron: invalid cron rule: %S" rule)
+           (message "[Cron-Repeat] Invalid cron rule: %s" rule)
            nil)))))
 
 (defun org-repeat-by-cron--reschedule-use-time-p (anchor-str cron-arity scheduled-str)
@@ -512,93 +550,116 @@ if any of the following conditions are met, in order of precedence:
 
 If none of these conditions are true, the function returns a
 date-only format string (\"%Y-%m-%d %a\")."
-  (let ((fmt-time  "%Y-%m-%d %a %H:%M")
-        (fmt-date  "%Y-%m-%d %a")
-        (time-regexp "[0-9]\\{2\\}:[0-9]\\{2\\}"))
-    (cond
-     ((eq cron-arity 5)
-      fmt-time)
-     ((eq cron-arity 3)
-      fmt-date)
-     (anchor-str
-      (if (string-match-p time-regexp anchor-str)
-          fmt-time
-        fmt-date))
-     (scheduled-str
-      (if (string-match-p time-regexp scheduled-str)
-          fmt-time
-        fmt-date))
-     (t fmt-date))))
+  (let ((time-regexp "[0-9]\\{2\\}:[0-9]\\{2\\}"))
+    (if (or (eq cron-arity 5)
+            (and anchor-str (string-match-p time-regexp anchor-str))
+            (and scheduled-str (string-match-p time-regexp scheduled-str)))
+        "%Y-%m-%d %a %H:%M"
+      "%Y-%m-%d %a")))
+
+(defun org-repeat-by-cron--extract-repeater (ts-str)
+  "Extract the Org repeater cookie from timestamp string TS-STR.
+
+Return the match (e.g., \"+1d\", \"++2w\", or \".1m/3d\") if found,
+otherwise nil."
+  (when (and ts-str
+             (string-match "\\s-*\\([.+]\\{1,2\\}[0-9]+[hdwmy]\\(?:/[0-9]+[hdwmy]\\)?\\)" ts-str))
+    (match-string 1 ts-str)))
 
 (defun org-repeat-by-cron-on-done (change-plist)
-  "Reschedule an Org entry using a cron rule when it is marked DONE.
+  "Reschedule an Org entry based on cron rules when it is marked DONE.
 
-This function is intended to be called from the hook
-`org-trigger-hook'.  When an entry is marked with
-a DONE state, this function calculates the next scheduled time
-based on a cron rule and updates the entry accordingly.
+This function is intended for use in `org-trigger-hook'.  It examines
+CHANGE-PLIST to determine if the task transitioned from a non-done
+state to a done state as defined by `org-done-keywords'.
 
-The behavior is controlled by properties within the entry's drawer:
-- The property stored in `org-repeat-by-cron-cron-prop' must contain
-  the cron string.  This is required.
-- The property in `org-repeat-by-cron-anchor-prop' stores the
-  base time for calculating the next occurrence.  This function
-  automatically updates this property to the new time after a
-  reschedule.
-- If the property in `org-repeat-by-cron-deadline-prop' is \"t\",
-  this function reschedules the `DEADLINE` timestamp and clears
-  the SCHEDULED one.  Otherwise, it reschedules SCHEDULED.
-- If the property in `org-repeat-by-cron-day-and-prop' is \"t\",
-  the day-of-month and day-of-week rules are combined with a
-  logical AND; otherwise, they are combined with OR.
+If the entry has a cron property defined by the user option
+`org-repeat-by-cron-cron-prop', this function calculates the next
+matching time using `org-repeat-by-cron-next-time'.  The calculation
+uses the following precedence for the base time:
+  1. The value of `org-repeat-by-cron-anchor-prop'.
+  2. The current SCHEDULED or DEADLINE timestamp.
+  3. The current system time.
 
-The base time for the calculation is chosen in the following order
-of precedence: the anchor property's time, the existing SCHEDULED
-or DEADLINE time, and finally the current time.
+Once the next time is determined, this function:
+  - Updates the anchor property with the new raw timestamp.
+  - Reschedules the entry using `org-schedule' or `org-deadline'.
+  - Reverts the entry to the `todo' state.
+  - Preserves existing Org repeater cookies (e.g., \\=\"+1d\\=\").
 
-Upon a successful reschedule, the entry's TODO state is reset to
-the first non-done state (TODO by default)."
-  (let ((from-state (format "%s" (plist-get change-plist :from)))
-        (to-state (format "%s" (plist-get change-plist :to))))
-    (when (and (not (member from-state org-done-keywords))
+If `org-repeat-by-cron-deadline-prop' is also configured, the
+function independently handles the deadline repetition logic.  If a
+valid time cannot be found within `org-repeat-by-cron-max-search-year',
+display a message and perform no changes."
+  (let* ((from-state (format "%s" (plist-get change-plist :from)))
+         (to-state (format "%s" (plist-get change-plist :to)))
+         (pom (point))
+         (cron-str (org-entry-get pom org-repeat-by-cron-cron-prop)))
+    (when (and cron-str
+               (not (string-empty-p (string-trim cron-str)))
+               (not (member from-state org-done-keywords))
                (member to-state org-done-keywords))
       (save-excursion
         (org-back-to-heading t)
-        (let* ((day-and-str   (org-entry-get (point) org-repeat-by-cron-day-and-prop nil))
-               (day-and-p     (if (string= day-and-str "t") t nil))
-               (cron-str       (org-entry-get (point) org-repeat-by-cron-cron-prop nil))
-               (deadline-str (org-entry-get (point) org-repeat-by-cron-deadline-prop nil))
-               (resched-func (if (string= deadline-str "t") #'org-deadline #'org-schedule))
-               (has-cron       (and cron-str     (> (length (string-trim cron-str))     0)))
-               (repeat-time     (if (string= deadline-str "t")  (org-get-deadline-time (point)) (org-get-scheduled-time (point))))
-               (anchor-str     (org-entry-get (point) org-repeat-by-cron-anchor-prop nil))
-               (anchor-time    (when (and anchor-str (> (length (string-trim anchor-str)) 0))
-                                 (org-time-string-to-time anchor-str)))
-               (base-time           (or anchor-time repeat-time (current-time)))
-               (cron-arity          (when has-cron     (org-repeat-by-cron--cron-rule-arity cron-str)))
-               (norm-cron      (when cron-arity        (org-repeat-by-cron--normalize-cron-rule cron-str)))
-               (resched-str  (org-entry-get (point) (if (string= deadline-str "t") "DEADLINE" "SCHEDULED")))
-               (fmt            (org-repeat-by-cron--reschedule-use-time-p
-                                anchor-str cron-arity resched-str)))
-          (when has-cron
-            (when (and has-cron (null cron-arity))
-              (message "[repeat] invalid cron rule, skipping"))
-            (when-let* ((bigger-time (if (time-less-p base-time nil) (current-time)
-                                       base-time))
-                        (next (org-repeat-by-cron-next-time norm-cron bigger-time day-and-p)))
-              (when next
-                (funcall resched-func nil (format-time-string fmt next))
-                ;; When using deadline, we need to manually clear SHCEDULED timestamp
-                (when (string= deadline-str "t") (org-schedule '(4)))
-                (org-entry-put (point) org-repeat-by-cron-anchor-prop
-                               (format-time-string fmt next))
+        (let ((cron-arity (org-repeat-by-cron--cron-rule-arity cron-str)))
+          (if (not cron-arity)
+              (message "[Cron-Repeat] Invalid cron rule: %s" cron-str)
+            (cl-labels ((update-anchor-return-cons (pom cron-str cron-arity anchor-str repeat-time resched-str)
+                          (let* ((day-and-p (string= (org-entry-get pom org-repeat-by-cron-day-and-prop) "t"))
+                                 (now (current-time))
+                                 (base-time (or (and anchor-str (org-time-string-to-time anchor-str))
+                                                repeat-time
+                                                now))
+                                 (next (org-repeat-by-cron-next-time (org-repeat-by-cron--normalize-cron-rule cron-str)
+                                                                     (if (time-less-p base-time now) now base-time)
+                                                                     day-and-p))
+                                 (fmt (org-repeat-by-cron--reschedule-use-time-p
+                                       anchor-str cron-arity resched-str))
+                                 (next-raw (format-time-string fmt next))
+                                 (repeater-cookie (org-repeat-by-cron--extract-repeater resched-str)))
+                            (if next
+                                (progn (org-entry-put pom org-repeat-by-cron-anchor-prop next-raw)
+                                       (cons next-raw
+                                        (if repeater-cookie
+                                           (concat next-raw " " repeater-cookie)
+                                         next-raw)))
+                              (message "[Cron-Repeat] Cannot find valid time before %s" (+ (nth 5 (decode-time)) 50))
+                              nil))))
+              (let* ((deadline-str (org-entry-get pom org-repeat-by-cron-deadline-prop))
+                     (deadline-p (string= deadline-str "t"))
+                     (repeat-time (if deadline-p (org-get-deadline-time pom) (org-get-scheduled-time pom)))
+                     (anchor-str (org-entry-get pom org-repeat-by-cron-anchor-prop))
+                     (resched-str (org-entry-get pom (if deadline-p "DEADLINE" "SCHEDULED")))
+                     (deadline-arity (org-repeat-by-cron--cron-rule-arity deadline-str))
+                     (result-cons (update-anchor-return-cons pom cron-str cron-arity anchor-str repeat-time resched-str))
+                     (next-raw (car result-cons))
+                     (final-ts (cdr result-cons)))
+                (if deadline-p
+                    (progn (org-deadline nil final-ts)
+                           (org-schedule '(4)))
+                  (org-schedule nil final-ts))
                 (org-todo 'todo)
-                (message "[repeat] repeated to %s%s"
-                         (format-time-string fmt next)
-                         (if has-cron ", reset state"))))))))))
+                (message "[Cron-Repeat] %s repeat to %s" (if deadline-p "DEADLINE" "SCHEDULED") next-raw)
+                (when deadline-arity
+                  (let* ((cron-str deadline-str)
+                         (cron-arity deadline-arity)
+                         (anchor-str (org-entry-get pom org-repeat-by-cron-deadline-anchor-prop))
+                         (repeat-time (org-get-deadline-time pom))
+                         (resched-str (org-entry-get pom "DEADLINE"))
+                         (org-repeat-by-cron-anchor-prop org-repeat-by-cron-deadline-anchor-prop)
+                         (result-cons (update-anchor-return-cons pom cron-str cron-arity anchor-str repeat-time resched-str))
+                         (next-raw (car result-cons))
+                         (final-ts (cdr result-cons)))
+                    (org-deadline nil final-ts)
+                    (message "[Cron-Repeat] DEADLINE repeat to %s" next-raw)))))))))))
+
 ;;;###autoload
 (define-minor-mode global-org-repeat-by-cron-mode
-  "A global minor mode globally enable org-repeat-by-cron."
+  "A global minor mode globally enable org-repeat-by-cron.
+
+When enabled, completing an Org task with a cron property will
+automatically reschedule it to the next matching occurrence by
+adding `org-repeat-by-cron-on-done' to `org-trigger-hook'."
   :init-value nil
   :global t
   :group 'org-repeat-by-cron
@@ -609,4 +670,3 @@ the first non-done state (TODO by default)."
 (provide 'org-repeat-by-cron)
 
 ;;; org-repeat-by-cron.el ends here
-
